/*
This software is available under NYSL(Nirunari Yakunari Sukinishiro License).
*/

/* 汎用の関数 */

/**
 * 10進数文字一桁を数値に変換 (サブルーチン)
 *
 * @param c 10進数文字
 * @return 0～9の数値 (エラー時は255)
 */
static unsigned char DecToDigit(char c)
{
    if( c>='0' && c<= '9'){
        return (unsigned char)(c - '0');

    }else{
        return 255;        /* エラー */
    }
}

/**
 * 0～9の数値を10進数文字に変換 (サブルーチン)
 *
 * @param val 0～9の数値
 * @return 10進数文字 (エラー時は'Z')
 */
static char DigitToDec(unsigned char val)
{
    if( val <= 9 ){
        return (char)('0' + val);

    }else{
        return 'Z';        /* エラー */
    }
}

/**
 * 10進数文字列を16ビット整数値に変換
 *
 * @param str 10進数文字列
 * @param val 変換した数値を返す
 * @param digit 桁数
 * @return 成否
 */
int DecToUint16(char *str, unsigned short *val, int digit)
{
int result = 0;
unsigned short acc = 0;
unsigned char  d;
int i;
    
    for(i=0;i<digit;i++){
        d = DecToDigit(str[i]);
        if(d > 9){
            result = -1;    /* エラー */
        }
        acc *= 10;
        acc += (unsigned short)d;
    }
    
    if(result == 0){
        *val = acc;
    }else{
        /* エラー時はvalを変更しない */
    }
    return result;
}

/**
 * 16ビット整数値を10進数文字列に変換
 *
 * @param str 変換した10進数文字列を返す
 * @param val 数値
 * @param digit 桁数
 */
void Uint16ToDec(char *str, unsigned short val, int digit)
{
int i;

    for(i=digit;i>0;i--){
        str[i-1] = DigitToDec((unsigned char)(val % 10));
        val /= 10;
    }
}


