/*
This software is available under NYSL(Nirunari Yakunari Sukinishiro License).
*/

/*
 * MyeongWol Controller Firmware
 * by Bizan Nishimura
 */
 
/*GR-SAKURA Sketch Template Version: V1.02*/
#include <rxduino.h>
#include <servo.h>
#include "steppers.h"
#include "common.h"
#include "tkusbhost.h"
#include "usb_bt.h"
#include "bt_spp.h"

/**
 * サーボ
 * [0]:Waist [1]:Neck [2]:Arm-L1 [3]:Arm-L2 [4]:Arm-R1 [5]:Arm-R2
 */
Servo servos[6];

/**
 * 初期化する
 */
void setup()
{
    // シリアル通信の設定
    // IO0(P21):RxD, IO1(P20):TxD */
//    Serial.begin(19200,SCI_SCI0P2x);
    //Serial.println("Annyeong haseyo!");

    pinMode(PIN_LED0,OUTPUT);
    pinMode(PIN_LED1,OUTPUT);
    pinMode(PIN_LED2,OUTPUT);
    pinMode(PIN_LED3,OUTPUT);
    digitalWrite(PIN_LED0, 0);
    digitalWrite(PIN_LED1, 0);
    digitalWrite(PIN_LED2, 0);
    digitalWrite(PIN_LED3, 0);

    // ステッピングモータの設定
    // IO6:nEnable, IO5:L-Clk, IO4:L-Dir, IO3:R-Clk, IO2:R-Dir
    steppers_init(6,5,4,3,2);
    steppers_enable();
    steppers_stop();
    
    // サーボの設定
    pinMode(7,OUTPUT); // IO7:Servo Enable (High Active)
    digitalWrite(7,  1);
    servos[0].attach(13); // IO13:Waist
    servos[1].attach(12); // IO12:Neck
    servos[2].attach(9);  // IO9:Arm-L1
    servos[3].attach(8);  // IO8:Arm-L2
    servos[4].attach(11); // IO11:Arm-R1
    servos[5].attach(10); // IO10:Arm-R2
    // サーボの初期角度
    servos[0].write(90);
    servos[1].write(90);
    servos[2].write(90);
    servos[3].write(90);
    servos[4].write(90);
    servos[5].write(90);
    
    delay(100);
    
    // USBの初期化
    if(tkusbh_init() != TKUSBH_OK)
    {
//        Serial.print("!!! USB Host Initialization ERROR !!! system halt\n");
        while(1);
    }
}

/**
 * 受信したコマンドの実行
 *
 * @param buff 受信したコマンドへのポインタ
 */
extern "C" void execute_command(char* buff)
{
    int servoNum;
    unsigned short val;
    static int toggle = 0;
    
    switch(buff[0])
    {
    /* サーボコマンド */
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
        //Serial.println("SERVO ");
        digitalWrite(PIN_LED1, toggle );
        toggle = 1 - toggle;
        /* サーボ番号 */
        servoNum = (int)(buff[0] - '1');
        /* 指令角度が有効(180以下)なら設定 */
        if( DecToUint16(&buff[1], &val, 3) == 0 )
        {
            if( val<=180 )
            {
                //Serial.println("DEG ");
                if(servoNum==3 || servoNum==4 || servoNum==5)
                {
                    val = 180 - val;
                }
                servos[servoNum].write(val);
            }
        }
        break;
    
    /* 前進コマンド */
    case 'F':
        //steppers_forward(2);
        /* 指令速度が有効(10以下)なら設定 */
        if( DecToUint16(&buff[1], &val, 2) == 0 )
        {
            if( val<=10 )
            {
                steppers_forward(val);
            }
        }
        break;
        
    /* 後退コマンド */
    case 'B':
        //steppers_backward(2);
        /* 指令速度が有効(10以下)なら設定 */
        if( DecToUint16(&buff[1], &val, 2) == 0 )
        {
            if( val<=10 )
            {
                steppers_backward(val);
            }
        }
        break;
        
    /* 左回転コマンド */
    case 'L':
        //steppers_turnLeft(2);
        /* 指令速度が有効(10以下)なら設定 */
        if( DecToUint16(&buff[1], &val, 2) == 0 )
        {
            if( val<=10 )
            {
                steppers_turnLeft(val);
            }
        }
        break;
        
    /* 右回転コマンド */
    case 'R':
        //steppers_turnRight(2);
        /* 指令速度が有効(10以下)なら設定 */
        if( DecToUint16(&buff[1], &val, 2) == 0 )
        {
            if( val<=10 )
            {
                steppers_turnRight(val);
            }
        }
        break;
    
    /* 停止コマンド */
    case 'S':
        steppers_stop();
        break;
    }
}

/**
 * メインループ
 */
void loop()
{
    /* USBタスク */
    usb_task();
    digitalWrite(PIN_LED2, 1);
    /* Bluetoothタスク */
    bt_task();
    digitalWrite(PIN_LED2, 0);
}


